//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : Timer
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Objects constructed from the Timer class act like tiny stopwatches
// (accurate to 1 millisecond). We can use them to determine the time it
// takes a program to run or the time it takes for users to respond to a
// question.   
//
// Future Plans   : Keep track of the number of on/off transitions
//                    during which < 1 millisecond elapses; such
//                    indicates trying to time something that elapses
//                    too quickly.
//
// Program History:
//   5/18/01: R. Pattis - Operational for 15-100
//   8/20/01: R. Pattis - JavaDoc comments added
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

package edu.cmu.cs.pattis.cs151xx;





/** 
 * Objects constructed from the Timer class act like tiny stopwatches
 *   (accurate to 1 millisecond).
 * We can use them to determine the time it takes a program to run or the
 *   time it takes for users to respond to a question.   
 * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
*/
public class Timer { 


	/** 
	 * Constructs a <code>Timer</code> object, stopped and showing no time
	 *   elapsed.
	*/
  public Timer ()
  {}
  

  
	/** 
	 * Reset this timer, so it is stopped and showing no time elapsed.
	*/
  public void reset ()
  {
    runningNow   = false;
    elapsedPrior = 0;
  }

  

	/** 
	 * Start this timer, so it is accumulating time (if it has already
	 *   been started, no changes are made).
	*/
  public void start ()
  {
    if (runningNow)
      return;
    
    runningNow    = true;
    lastStartTime = System.currentTimeMillis();
  }

  
  
	/** 
	 * Stop this timer, so it is not accumulating time (if it has already
	 *   been stopped, no changes are made).
	*/
  public void stop ()
  {
    if (!runningNow)
      return;
      
    runningNow    = false;
    elapsedPrior += (System.currentTimeMillis() - lastStartTime);
      
  }


  
	/** 
	 * Return the amount of elapsed time; this method returns the correct
	 *   result whether the time is started or stopped.
	 *
	 * @return the amount of elapsed time (in seconds)
	*/
  public double getElapsed()
  {return ((double)(elapsedPrior + (runningNow ? System.currentTimeMillis()-lastStartTime : 0)))/1000.;}

  

	/** 
	 * Returns a String representation of the state of this timer.
	 *
	 * @return (1) whether the timer is running, (2) how much time elapsed
	 *         before it was most recently started, and (3) how much time
	 *         has elapsed since it was most recently started
	*/
  public String toString ()
  {return "Timer[running="+runningNow+"/"+
          "elapsedPrior="+elapsedPrior+"/"+
          "deltaWhileRunning="+(runningNow ? System.currentTimeMillis()-lastStartTime : 0) +
          "]";
  }
  
  
  
  //No JavaDoc comments for private instance variables
  
  private boolean runningNow;
  private long    elapsedPrior;
  private long    lastStartTime;
}




