//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : TypedBufferWriter
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
// Objects constructed from the TypedBufferWriter class allow primitive
// types and strings to be written into files. Most every reference type
// has some string representation (accessible via its toString method).   
//
// Future Plans   : None
//
// Program History:
//   9/12/01: R. Pattis - Operational for 15-100
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;


import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;





/** 
 * Objects constructed from the TypedBufferWriter class allow primitive
 *   types and strings to be written into files.
 * Most every reference type has some string representation
 *   (accessible via its <code>toString</code> method).   
 * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
*/
public class TypedBufferWriter {
  

	/** 
	 * Constructs a <code>TypedBufferWriter</code> object, for writing into
	 *   the specifed file name.
	 *
	 * @param  fileName specifies the fileName into which this object writes
	*/
  public TypedBufferWriter (String fileName)
  {
    this.fileName = stripQuotes(fileName);
    for (;;)
      try {
        bw = new PrintWriter(new FileOutputStream(fileName), /*autoFlush*/true);
        break;
      }catch (FileNotFoundException e)
         {fileName = Prompt.forString("Unable to open file named \"" + fileName + "; re-enter");}
  }
  
  

	/** 
	 * Constructs a <code>TypedBufferWriter</code> object, for writing onto
	 *   the specifed <code>OutputStream</code> (e.g., <code>System.out</code>).
	 *
	 * @param  os specifies an output stream to write to (e.g.,
	 *         <code>System.out</code> to write to the console)
	*/
  public TypedBufferWriter (OutputStream os)
  {
    fileName = "some.OutputStream"; 
    bw       = new PrintWriter( new OutputStreamWriter(os), /*autoFlush*/true);
  }
  
  
    
  
	/** 
	 * Returns the file name that this this object is connected to.
	 *
	 * @return the file name that this this object is connected to (or
	 *         "some-OutputStream")
	*/
  public String getFileName ()
  {return fileName;}
  
  
  
  
  
	/** 
	 * Prints an <code>int</code>.
	 *
	 * @param o specifies the value to print
	*/
  public void print (int o)
  {bw.print(o);}
     
 
   
    

	/** 
	 * Prints an <code>int</code> followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (int o)
  {bw.println(o);}
     
 
   
    

	/** 
	 * Prints a <code>double</code>.
	 *
	 * @param o specifies the value to print
	*/
  public void print (double o)
  {bw.print(o);}
     
 
   
    

	/** 
	 * Prints a <code>double</code> followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (double o)
  {bw.println(o);}
     
 
   
    

	/** 
	 * Prints a <code>boolean</code>.
	 *
	 * @param o specifies the value to print
	*/
  public void print (boolean o)
  {bw.print(o);}
     
 
   
    

	/** 
	 * Prints an <code>int</code> followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (boolean o)
  {bw.println(o);}
     
 
   
    

	/** 
	 * Prints a <code>char</code>.
	 *
	 * @param o specifies the value to print
	*/
  public void print (char o)
  {bw.print(o);}
     
 
   
    

	/** 
	 * Prints an <code>char</code> followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (char o)
  {bw.println(o);}
     
 
   
    

	/** 
	 * Prints a <code>String</code>.
	 *
	 * @param o specifies the value to print
	*/
  public void print (String o)
  {bw.print(o);}
     
 
   
    

	/** 
	 * Prints an <code>String</code> followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (String o)
  {bw.println(o);}
     
 
   
    

	/** 
	 * Prints any <code>Object</code> (with a <code>toString</b> method).
	 *
	 * @param o specifies the value to print
	*/
  public void print (Object o)
  {bw.print(o.toString());}
     
 
   
    

	/** 
	 * Prints any <code>Object</code> (with a <code>toString</b> method)
   * followed by a new line.
	 *
	 * @param o specifies the value to print
	*/
  public void println (Object o)
  {bw.println(o.toString());}
     
 
   
    

	/** 
	 * Prints  a new line.
	*/
  public void println ()
  {bw.println();}
     
 
   
    

	/** 
	 * Closes an open <code>TypedBufferReader</code>, disconnecting it
	 *   from its source.
	*/
  public void close() 
  {bw.close();}
 
 
 
 	/** 
	 * Strips all the quotes from its argument.
	 *
	 * @param  s specifies a file name
	 *
	 * @return  a <code>String</code> containing all characters in <code>s</code> except the
	 *          quotes
	*/
  private static String stripQuotes(String s) 
  {
    String answer = "";
    for (int i=0; i<s.length(); i++)
      if (s.charAt(i) != '"')
        answer += s.charAt(i);
        
    return answer;
  }
 
 




	/** 
	 * Buffered reader (set to a file or an <code>InputStream</code>).
	*/
  private PrintWriter bw;


	/** 
	 * Name of file being processed.
	*/
  private String fileName;
  
}


